
"""
   TCP/IP versions of DOPY protocol objects.
"""

import dopy

import socket, thread, select, string
from pickle import Pickler, Unpickler

class Socket:
   
   """
      Provides the FileEventSource interface for sockets.
   """
   
   def fileno(self):
      return self.sock.fileno()
   
   def isActive(self):
      return self.sock is not None
   
   def shutdown(self):
      self.sock = None      


class Protocol(Socket, dopy.Protocol):
   
   def __init__(self, host, port, sock = None):
      hub = dopy.getHub()
      
      # try to convert the dotted ip address to the full address
      try:
         host = socket.gethostbyaddr(host)[0]
      except socket.error, ex:
         pass
      
      self.host = host
      self.port = port
      
      # if a socket was provided, use it.  Otherwise create and connect one.
      if sock:
         self.sock = sock
      else:
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.connect(host, port)
         
      self.file = self.sock.makefile('r+')
      if hub.isThreaded():
         self.__lk = thread.allocate_lock()
      hub.addProtocol(self)

   def send(self, obj):   
#      print 'sending %s on tcp:%s:%s:%d' % \
#         (obj, self.host, self.port, self.sock.fileno())
      hub = dopy.getHub()
      if hub.isThreaded():
         self.__lk.acquire()
#         print '   got lock'
      try:
         Pickler(self.file).dump(obj)
         self.file.flush()
      finally:
         if hub.isThreaded():
            self.__lk.release()
#            print '   released lock'

   def getNext(self):
      # try to unpickle something off of the stream, return None if we
      # terminate.
      try:
#         print 'waiting for request on tcp:%s:%s:%d' % (self.host, self.port,
#                                                        self.sock.fileno()
#                                                        )
                                                        
         # wait till there's an error or something to read
         rdx, wrx, erx = select.select([self.sock], [], [self.sock])
         evt =  Unpickler(self.file).load()
      except EOFError:
         return None
#      print 'got %s on tcp:%s:%s:%d' % (evt, self.host, self.port,
#                                        self.sock.fileno()
#                                        )
      return evt

   def getProtocolId(self):
      return 'tcp:%s:%d' % (self.host, self.port)
   
   def __getstate__(self):
      return { 'peer': self.getpeername() }
   
   def __setstate__(self, dict):
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
      peer = dict['peer']
      self.sock.connect(peer[0], peer[1])
      self.file = self.sock.makefile('r+')

class Server(Socket, dopy.ProtocolServer):
   
   def __init__(self, port):
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      self.sock.bind('', port)
      self.sock.listen(5)

   def acceptConnection(self):
      sock, addr = self.sock.accept()      
      return Protocol(addr[0], addr[1], sock = sock)

def _factory(id):
   """
      Factory to create TCP protocols.
   """
   type, host, port = id
   port = string.atoi(port)
   return Protocol(host, port)   
   
def makeServer(port):
   hub = dopy.getHub()
   hub.addServer(Server(port))

def remote(host, port, key):
   hub = dopy.getHub()
   prot = hub.getProtocol('tcp:%s:%d' % (host, port))
   return dopy.DynamicRemoteObject(prot, key)

# register the factory
dopy.protocolFactory.register('tcp', _factory)
